@@ -15,18 +15,16 @@
 import os
 import io
 import abc
-import six
 import math
+from typing import Union
 import numpy as np
 import librosa
 import soundfile as sf
 import tensorflow as tf
 import tensorflow_io as tfio
 
-from ..utils.utils import log10, has_tpu
-from .gammatone import fft_weights
-
-tpu = has_tpu()
+from ..utils import math_util, env_util
+from .methods import gammatone
 
 
 # def tf_resample(signal, rate_in, rate_out):
@@ -42,7 +40,7 @@ def load_and_convert_to_wav(path: str) -> tf.Tensor:
     return tf.audio.encode_wav(tf.expand_dims(wave, axis=-1), sample_rate=rate)
 
 
-def read_raw_audio(audio, sample_rate=16000):
+def read_raw_audio(audio: Union[str, bytes, np.ndarray], sample_rate=16000) -> np.ndarray:
     if isinstance(audio, str):
         wave, _ = librosa.load(os.path.expanduser(audio), sr=sample_rate, mono=True)
     elif isinstance(audio, bytes):
@@ -58,9 +56,9 @@ def read_raw_audio(audio, sample_rate=16000):
     return wave
 
 
-def tf_read_raw_audio(audio: tf.Tensor, sample_rate=16000):
+def tf_read_raw_audio(audio: tf.Tensor, sample_rate=16000) -> tf.Tensor:
     wave, rate = tf.audio.decode_wav(audio, desired_channels=1, desired_samples=-1)
-    if not tpu:
+    if not env_util.has_devices("TPU"):
         resampled = tfio.audio.resample(wave, rate_in=tf.cast(rate, dtype=tf.int64), rate_out=sample_rate)
         return tf.reshape(resampled, shape=[-1])  # reshape for using tf.signal
     return tf.reshape(wave, shape=[-1])  # reshape for using tf.signal
@@ -94,16 +92,16 @@ def merge_slices(slices: np.ndarray) -> np.ndarray:
     return np.reshape(slices, [-1])
 
 
-def normalize_audio_feature(audio_feature: np.ndarray, per_feature=False):
+def normalize_audio_feature(audio_feature: np.ndarray, per_frame=False) -> np.ndarray:
     """ Mean and variance normalization """
-    axis = 0 if per_feature else None
+    axis = 1 if per_frame else None
     mean = np.mean(audio_feature, axis=axis)
-    std_dev = np.std(audio_feature, axis=axis) + 1e-9
+    std_dev = np.sqrt(np.var(audio_feature, axis=axis) + 1e-9)
     normalized = (audio_feature - mean) / std_dev
     return normalized
 
 
-def tf_normalize_audio_features(audio_feature: tf.Tensor, per_feature=False):
+def tf_normalize_audio_features(audio_feature: tf.Tensor, per_frame=False) -> tf.Tensor:
     """
     TF Mean and variance features normalization
     Args:
@@ -112,19 +110,19 @@ def tf_normalize_audio_features(audio_feature: tf.Tensor, per_feature=False):
     Returns:
         normalized audio features with shape [T, F]
     """
-    axis = 0 if per_feature else None
-    mean = tf.reduce_mean(audio_feature, axis=axis)
-    std_dev = tf.math.reduce_std(audio_feature, axis=axis) + 1e-9
+    axis = 1 if per_frame else None
+    mean = tf.reduce_mean(audio_feature, axis=axis, keepdims=True)
+    std_dev = tf.math.sqrt(tf.math.reduce_variance(audio_feature, axis=axis, keepdims=True) + 1e-9)
     return (audio_feature - mean) / std_dev
 
 
-def normalize_signal(signal: np.ndarray):
+def normalize_signal(signal: np.ndarray) -> np.ndarray:
     """ Normailize signal to [-1, 1] range """
     gain = 1.0 / (np.max(np.abs(signal)) + 1e-9)
     return signal * gain
 
 
-def tf_normalize_signal(signal: tf.Tensor):
+def tf_normalize_signal(signal: tf.Tensor) -> tf.Tensor:
     """
     TF Normailize signal to [-1, 1] range
     Args:
@@ -137,7 +135,7 @@ def tf_normalize_signal(signal: tf.Tensor):
     return signal * gain
 
 
-def preemphasis(signal: np.ndarray, coeff=0.97):
+def preemphasis(signal: np.ndarray, coeff=0.97) -> np.ndarray:
     if not coeff or coeff <= 0.0:
         return signal
     return np.append(signal[0], signal[1:] - coeff * signal[:-1])
@@ -159,7 +157,7 @@ def tf_preemphasis(signal: tf.Tensor, coeff=0.97):
     return tf.concat([s0, s1], axis=-1)
 
 
-def depreemphasis(signal: np.ndarray, coeff=0.97):
+def depreemphasis(signal: np.ndarray, coeff=0.97) -> np.ndarray:
     if not coeff or coeff <= 0.0: return signal
     x = np.zeros(signal.shape[0], dtype=np.float32)
     x[0] = signal[0]
@@ -168,7 +166,7 @@ def depreemphasis(signal: np.ndarray, coeff=0.97):
     return x
 
 
-def tf_depreemphasis(signal: tf.Tensor, coeff=0.97):
+def tf_depreemphasis(signal: tf.Tensor, coeff=0.97) -> tf.Tensor:
     """
     TF Depreemphasis
     Args:
@@ -206,7 +204,7 @@ class SpeechFeaturizer(metaclass=abc.ABCMeta):
             "pitch": bool,
             "normalize_signal": bool,
             "normalize_feature": bool,
-            "normalize_per_feature": bool
+            "normalize_per_frame": bool
         }
         """
         # Samples
@@ -217,10 +215,11 @@ class SpeechFeaturizer(metaclass=abc.ABCMeta):
         self.num_feature_bins = speech_config.get("num_feature_bins", 80)
         self.feature_type = speech_config.get("feature_type", "log_mel_spectrogram")
         self.preemphasis = speech_config.get("preemphasis", None)
+        self.top_db = speech_config.get("top_db", 80.0)
         # Normalization
         self.normalize_signal = speech_config.get("normalize_signal", True)
         self.normalize_feature = speech_config.get("normalize_feature", True)
-        self.normalize_per_feature = speech_config.get("normalize_per_feature", False)
+        self.normalize_per_frame = speech_config.get("normalize_per_frame", False)
         self.center = speech_config.get("center", True)
         # Length
         self.max_length = 0
@@ -315,28 +314,28 @@ class NumpySpeechFeaturizer(SpeechFeaturizer):
         original_features = features.copy()
 
         if self.normalize_feature:
-            features = normalize_audio_feature(features, per_feature=self.normalize_per_feature)
+            features = normalize_audio_feature(features, per_frame=self.normalize_per_frame)
 
         features = np.expand_dims(features, axis=-1)
 
         if self.delta:
             delta = librosa.feature.delta(original_features.T).T
             if self.normalize_feature:
-                delta = normalize_audio_feature(delta, per_feature=self.normalize_per_feature)
+                delta = normalize_audio_feature(delta, per_frame=self.normalize_per_frame)
             features = np.concatenate([features, np.expand_dims(delta, axis=-1)], axis=-1)
 
         if self.delta_delta:
             delta_delta = librosa.feature.delta(original_features.T, order=2).T
             if self.normalize_feature:
                 delta_delta = normalize_audio_feature(
-                    delta_delta, per_feature=self.normalize_per_feature)
+                    delta_delta, per_frame=self.normalize_per_frame)
             features = np.concatenate([features, np.expand_dims(delta_delta, axis=-1)], axis=-1)
 
         if self.pitch:
             pitches = self.compute_pitch(signal)
             if self.normalize_feature:
                 pitches = normalize_audio_feature(
-                    pitches, per_feature=self.normalize_per_feature)
+                    pitches, per_frame=self.normalize_per_frame)
             features = np.concatenate([features, np.expand_dims(pitches, axis=-1)], axis=-1)
 
         return features
@@ -398,16 +397,16 @@ class NumpySpeechFeaturizer(SpeechFeaturizer):
     def compute_log_gammatone_spectrogram(self, signal: np.ndarray) -> np.ndarray:
         S = self.stft(signal)
 
-        gammatone = fft_weights(self.nfft, self.sample_rate,
-                                self.num_feature_bins, width=1.0,
-                                fmin=0, fmax=int(self.sample_rate / 2),
-                                maxlen=(self.nfft / 2 + 1))
+        gtone = gammatone.fft_weights(self.nfft, self.sample_rate,
+                                      self.num_feature_bins, width=1.0,
+                                      fmin=0, fmax=int(self.sample_rate / 2),
+                                      maxlen=(self.nfft / 2 + 1))
 
-        gammatone = gammatone.numpy().astype(np.float32)
+        gtone = gtone.numpy().astype(np.float32)
 
-        gammatone_spectrogram = np.dot(S.T, gammatone)
+        gtone_spectrogram = np.dot(S.T, gtone)
 
-        return self.power_to_db(gammatone_spectrogram)
+        return self.power_to_db(gtone_spectrogram)
 
 
 class TFSpeechFeaturizer(SpeechFeaturizer):
@@ -426,25 +425,14 @@ class TFSpeechFeaturizer(SpeechFeaturizer):
         framed_signals *= window
         return tf.square(tf.abs(tf.signal.rfft(framed_signals, [self.nfft])))
 
-    def power_to_db(self, S, ref=1.0, amin=1e-10, top_db=80.0):
-        if amin <= 0:
-            raise ValueError('amin must be strictly positive')
+    def power_to_db(self, S, amin=1e-10):
+        log_spec = 10.0 * math_util.log10(tf.maximum(amin, S))
+        log_spec -= 10.0 * math_util.log10(tf.maximum(amin, 1.0))
 
-        magnitude = S
-
-        if six.callable(ref):
-            # User supplied a function to calculate reference power
-            ref_value = ref(magnitude)
-        else:
-            ref_value = np.abs(ref)
-
-        log_spec = 10.0 * log10(tf.maximum(amin, magnitude))
-        log_spec -= 10.0 * log10(tf.maximum(amin, ref_value))
-
-        if top_db is not None:
-            if top_db < 0:
+        if self.top_db is not None:
+            if self.top_db < 0:
                 raise ValueError('top_db must be non-negative')
-            log_spec = tf.maximum(log_spec, tf.reduce_max(log_spec) - top_db)
+            log_spec = tf.maximum(log_spec, tf.reduce_max(log_spec) - self.top_db)
 
         return log_spec
 
@@ -480,7 +468,7 @@ class TFSpeechFeaturizer(SpeechFeaturizer):
         features = tf.expand_dims(features, axis=-1)
 
         if self.normalize_feature:
-            features = tf_normalize_audio_features(features, per_feature=self.normalize_per_feature)
+            features = tf_normalize_audio_features(features, per_frame=self.normalize_per_frame)
 
         return features
 
@@ -507,11 +495,11 @@ class TFSpeechFeaturizer(SpeechFeaturizer):
     def compute_log_gammatone_spectrogram(self, signal: np.ndarray) -> np.ndarray:
         S = self.stft(signal)
 
-        gammatone = fft_weights(self.nfft, self.sample_rate,
-                                self.num_feature_bins, width=1.0,
-                                fmin=0, fmax=int(self.sample_rate / 2),
-                                maxlen=(self.nfft / 2 + 1))
+        gtone = gammatone.fft_weights(self.nfft, self.sample_rate,
+                                      self.num_feature_bins, width=1.0,
+                                      fmin=0, fmax=int(self.sample_rate / 2),
+                                      maxlen=(self.nfft / 2 + 1))
 
-        gammatone_spectrogram = tf.tensordot(S, gammatone, 1)
+        gtone_spectrogram = tf.tensordot(S, gtone, 1)
 
-        return self.power_to_db(gammatone_spectrogram)
+        return self.power_to_db(gtone_spectrogram)
